/**
 * Satisfy typescript compiler about universal decorators.
 */
export declare type ParamOrPropDecorator = (object: object, propertyName: string, index?: number) => void;
/**
 * Allows to inject a custom repository using TypeDI's Container.
 * Be aware that you have to annotate the param/property  with correct type!
 * ```ts
 * class Sample {
 *   // constructor injection
 *   constructor(
 *     \@OrmRepository()
 *      private userRepository: UserRepository,
 *   ) {}
 *
 *   // property injection
 *  \@OrmRepository()
 *   userRepository: UserRepository;
 * }
 * ```
 */
export declare function OrmRepository(): ParamOrPropDecorator;
/**
 * Allows to inject a Repository, MongoRepository, TreeRepository using TypeDI's Container.
 * Be aware that you have to annotate the param/property  with correct type!
 * ```ts
 * class Sample {
 *   // constructor injection
 *   constructor(
 *     \@OrmRepository(User)
 *      private userRepository: Repository<User>,
 *   ) {}
 *
 *   // property injection
 *  \@OrmRepository(User)
 *   userRepository: Repository<User>;
 * }
 * ```
 */
export declare function OrmRepository(entityType: Function): ParamOrPropDecorator;
/**
 * Allows to inject a custom repository using TypeDI's Container
 * and specify the connection name in a parameter.
 * Be aware that you have to annotate the param/property  with correct type!
 * ```ts
 * class Sample {
 *   // constructor injection
 *   constructor(
 *     \@OrmRepository("test-conn")
 *      private userRepository: UserRepository,
 *   ) {}
 *
 *   // property injection
 *  \@OrmRepository("test-conn")
 *   userRepository: UserRepository;
 * }
 * ```
 */
export declare function OrmRepository(connectionName: string): ParamOrPropDecorator;
/**
 * Allows to inject a Repository, MongoRepository, TreeRepository using TypeDI's Container
 * and specify the connection name in a parameter.
 * Be aware that you have to annotate the param/property with correct type!
 * ```ts
 * class Sample {
 *   // constructor injection
 *   constructor(
 *     \@OrmRepository(User, "test-conn")
 *      private userRepository: Repository<User>,
 *   ) {}
 *
 *   // property injection
 *  \@OrmRepository(User, "test-conn")
 *   userRepository: Repository<User>;
 * }
 * ```
 */
export declare function OrmRepository(entityType: Function, connectionName: string): ParamOrPropDecorator;
